@@ -87,7 +87,7 @@ AltTabPopup.prototype = {
         let [childMinHeight, childNaturalHeight] = this._appSwitcher.actor.get_preferred_height(primary.width - hPadding);
         let [childMinWidth, childNaturalWidth] = this._appSwitcher.actor.get_preferred_width(childNaturalHeight);
         childBox.x1 = Math.max(primary.x + leftPadding, primary.x + Math.floor((primary.width - childNaturalWidth) / 2));
-        childBox.x2 = Math.min(primary.x + primary.width - hPadding, childBox.x1 + childNaturalWidth);
+        childBox.x2 = Math.min(primary.x + primary.width - rightPadding, childBox.x1 + childNaturalWidth);
         childBox.y1 = primary.y + Math.floor((primary.height - childNaturalHeight) / 2);
         childBox.y2 = childBox.y1 + childNaturalHeight;
         this._appSwitcher.actor.allocate(childBox, flags);
@@ -97,8 +97,6 @@ AltTabPopup.prototype = {
         // those calculations
         if (this._thumbnails) {
             let icon = this._appIcons[this._currentApp].actor;
-            // Force a stage relayout to make sure we get the correct position
-            global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, 0, 0);
             let [posX, posY] = icon.get_transformed_position();
             let thumbnailCenter = posX + icon.width / 2;
             let [childMinWidth, childNaturalWidth] = this._thumbnails.actor.get_preferred_width(-1);
@@ -145,6 +143,12 @@ AltTabPopup.prototype = {
 
         this._appIcons = this._appSwitcher.icons;
 
+        // Need to force an allocation so we can figure out whether we
+        // need to scroll when selecting
+        this.actor.opacity = 0;
+        this.actor.show();
+        this.actor.get_allocation_box();
+
         // Make the initial selection
         if (switch_group) {
             if (backward) {
@@ -174,8 +178,6 @@ AltTabPopup.prototype = {
             return false;
         }
 
-        this.actor.opacity = 0;
-        this.actor.show();
         Tweener.addTween(this.actor,
                          { opacity: 255,
                            time: POPUP_FADE_TIME,
@@ -215,33 +217,21 @@ AltTabPopup.prototype = {
 
         this._disableHover();
 
-        if (action == Meta.KeyBindingAction.SWITCH_GROUP)
-            this._select(this._currentApp, backwards ? this._previousWindow() : this._nextWindow());
-        else if (keysym == Clutter.Escape)
+        if (keysym == Clutter.Escape) {
             this.destroy();
-        else if (this._thumbnailsFocused) {
-            if (action == Meta.KeyBindingAction.SWITCH_WINDOWS)
-                if (backwards) {
-                    if (this._currentWindow == 0 || this._currentWindow == -1)
-                        this._select(this._previousApp());
-                    else
-                        this._select(this._currentApp, this._previousWindow());
-                } else {
-                    if (this._currentWindow == this._appIcons[this._currentApp].cachedWindows.length - 1)
-                        this._select(this._nextApp());
-                    else
-                        this._select(this._currentApp, this._nextWindow());
-                }
-            else if (keysym == Clutter.Left)
+        } else if (action == Meta.KeyBindingAction.SWITCH_GROUP) {
+            this._select(this._currentApp, backwards ? this._previousWindow() : this._nextWindow());
+        } else if (action == Meta.KeyBindingAction.SWITCH_WINDOWS) {
+            this._select(backwards ? this._previousApp() : this._nextApp());
+        } else if (this._thumbnailsFocused) {
+            if (keysym == Clutter.Left)
                 this._select(this._currentApp, this._previousWindow());
             else if (keysym == Clutter.Right)
                 this._select(this._currentApp, this._nextWindow());
             else if (keysym == Clutter.Up)
                 this._select(this._currentApp, null, true);
         } else {
-            if (action == Meta.KeyBindingAction.SWITCH_WINDOWS)
-                this._select(backwards ? this._previousApp() : this._nextApp());
-            else if (keysym == Clutter.Left)
+            if (keysym == Clutter.Left)
                 this._select(this._previousApp());
             else if (keysym == Clutter.Right)
                 this._select(this._nextApp());
@@ -583,7 +573,7 @@ SwitcherList.prototype = {
         this._rightArrow.opacity = this._rightGradient.opacity;
     },
 
-    addItem : function(item) {
+    addItem : function(item, label) {
         let bbox = new St.Button({ style_class: 'item-box',
                                    reactive: true });
 
@@ -594,6 +584,8 @@ SwitcherList.prototype = {
         bbox.connect('clicked', Lang.bind(this, function() { this._onItemClicked(n); }));
         bbox.connect('enter-event', Lang.bind(this, function() { this._onItemEnter(n); }));
 
+        bbox.label_actor = label;
+
         this._items.push(bbox);
     },
 
@@ -979,7 +971,7 @@ AppSwitcher.prototype = {
 
     _addIcon : function(appIcon) {
         this.icons.push(appIcon);
-        this.addItem(appIcon.actor);
+        this.addItem(appIcon.actor, appIcon.label);
 
         let n = this._arrows.length;
         let arrow = new St.DrawingArea({ style_class: 'switcher-arrow' });
@@ -1049,9 +1041,12 @@ ThumbnailList.prototype = {
                 this._labels.push(bin);
                 bin.add_actor(name);
                 box.add_actor(bin);
+
+                this.addItem(box, name);
+            } else {
+                this.addItem(box, null);
             }
 
-            this.addItem(box);
         }
     },
 
