@@ -9,8 +9,6 @@ const Signals = imports.signals;
 const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 const Mainloop = imports.mainloop;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const AppFavorites = imports.ui.appFavorites;
 const DND = imports.ui.dnd;
@@ -46,7 +44,7 @@ AlphabeticalView.prototype = {
         this.actor = new St.ScrollView({ x_fill: true,
                                          y_fill: false,
                                          y_align: St.Align.START,
-                                         vfade: true });
+                                         style_class: 'vfade' });
         this.actor.add_actor(box);
         this.actor.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         this.actor.connect('notify::mapped', Lang.bind(this,
@@ -126,7 +124,7 @@ AlphabeticalView.prototype = {
             if (currentTimeMillis - startTimeMillis > MAX_APPLICATION_WORK_MILLIS)
                 break;
         }
-        this._pendingAppIds.splice(0, i);
+        this._pendingAppIds.splice(0, i + 1);
         if (this._pendingAppIds.length > 0) {
             return true;
         } else {
@@ -434,6 +432,8 @@ AppWellIcon.prototype = {
         this.icon = new AppIcon(app, iconParams);
         this.actor.set_child(this.icon.actor);
 
+        this.actor.label_actor = this.icon.label;
+
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
         this.actor.connect('clicked', Lang.bind(this, this._onClicked));
         this.actor.connect('popup-menu', Lang.bind(this, this._onKeyboardPopupMenu));
