@@ -5,14 +5,13 @@ const Cogl = imports.gi.Cogl;
 const GConf = imports.gi.GConf;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
+const Gtk = imports.gi.Gtk;
 const Pango = imports.gi.Pango;
 const St = imports.gi.St;
 const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const History = imports.misc.history;
 const ExtensionSystem = imports.ui.extensionSystem;
@@ -100,12 +99,19 @@ Notebook.prototype = {
     selectIndex: function(index) {
         if (index == this._selectedIndex)
             return;
-        this._unselect();
         if (index < 0) {
+            this._unselect();
             this.emit('selection', null);
             return;
         }
+
+        // Focus the new tab before unmapping the old one
         let tabData = this._tabs[index];
+        if (!tabData.scrollView.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false))
+            this.actor.grab_key_focus();
+
+        this._unselect();
+
         tabData.labelBox.add_style_pseudo_class('selected');
         tabData.scrollView.show();
         this._selectedIndex = index;
@@ -576,6 +582,53 @@ ErrorLog.prototype = {
     }
 };
 
+function Memory() {
+    this._init();
+}
+
+Memory.prototype = {
+    _init: function() {
+        this.actor = new St.BoxLayout({ vertical: true });
+        this._glibc_uordblks = new St.Label();
+        this.actor.add(this._glibc_uordblks);
+
+        this._js_bytes = new St.Label();
+        this.actor.add(this._js_bytes);
+
+        this._gjs_boxed = new St.Label();
+        this.actor.add(this._gjs_boxed);
+
+        this._gjs_gobject = new St.Label();
+        this.actor.add(this._gjs_gobject);
+
+        this._gjs_function = new St.Label();
+        this.actor.add(this._gjs_function);
+
+        this._gjs_closure = new St.Label();
+        this.actor.add(this._gjs_closure);
+
+        this._gcbutton = new St.Button({ label: 'Full GC',
+                                         style_class: 'lg-obj-inspector-button' });
+        this._gcbutton.connect('clicked', Lang.bind(this, function () { global.gc(); this._renderText(); }));
+        this.actor.add(this._gcbutton, { x_align: St.Align.START,
+                                         x_fill: false });
+
+        this.actor.connect('notify::mapped', Lang.bind(this, this._renderText));
+    },
+
+    _renderText: function() {
+        if (!this.actor.mapped)
+            return;
+        let memInfo = global.get_memory_info();
+        this._glibc_uordblks.text = 'glibc_uordblks: ' + memInfo.glibc_uordblks;
+        this._js_bytes.text = 'js bytes: ' + memInfo.js_bytes;
+        this._gjs_boxed.text = 'gjs_boxed: ' + memInfo.gjs_boxed;
+        this._gjs_gobject.text = 'gjs_gobject: ' + memInfo.gjs_gobject;
+        this._gjs_function.text = 'gjs_function: ' + memInfo.gjs_function;
+        this._gjs_closure.text = 'gjs_closure: ' + memInfo.gjs_closure;
+    }
+};
+
 function Extensions() {
     this._init();
 }
@@ -745,12 +798,7 @@ LookingGlass.prototype = {
         let label = new St.Label({ text: 'js>>> ' });
         entryArea.add(label);
 
-        this._entry = new St.Entry();
-        /* unmapping the edit box will un-focus it, undo that */
-        notebook.connect('selection', Lang.bind(this, function (nb, child) {
-            if (child == this._evalBox)
-                global.stage.set_key_focus(this._entry);
-        }));
+        this._entry = new St.Entry({ can_focus: true });
         entryArea.add(this._entry, { expand: true });
 
         this._windowList = new WindowList();
@@ -763,6 +811,9 @@ LookingGlass.prototype = {
         this._errorLog = new ErrorLog();
         notebook.appendPage('Errors', this._errorLog.actor);
 
+        this._memory = new Memory();
+        notebook.appendPage('Memory', this._memory.actor);
+
         this._extensions = new Extensions();
         notebook.appendPage('Extensions', this._extensions.actor);
 
@@ -909,6 +960,7 @@ LookingGlass.prototype = {
         if (!Main.pushModal(this._entry))
             return;
 
+        this._notebook.selectIndex(0);
         this.actor.show();
         this.actor.lower(Main.chrome.actor);
         this._open = true;
