@@ -1,11 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-imports.gi.versions.Clutter = '1.0';
-imports.gi.versions.Gio = '2.0';
-imports.gi.versions.Gdk = '3.0';
-imports.gi.versions.GdkPixbuf = '2.0';
-imports.gi.versions.Gtk = '3.0';
-
 const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
 const Gdk = imports.gi.Gdk;
@@ -17,8 +11,6 @@ const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const Chrome = imports.ui.chrome;
 const CtrlAltTab = imports.ui.ctrlAltTab;
@@ -75,17 +67,15 @@ let _cssStylesheet = null;
 let background = null;
 
 function start() {
-    // Add a binding for 'global' in the global JS namespace; (gjs
-    // keeps the web browser convention of having that namespace be
-    // called 'window'.)
-    window.global = Shell.Global.get();
-
-    // Now monkey patch utility functions into the global proxy;
+    // Monkey patch utility functions into the global proxy;
     // This is easier and faster than indirecting down into global
     // if we want to call back up into JS.
     global.logError = _logError;
     global.log = _logDebug;
 
+    // Chain up async errors reported from C
+    global.connect('notify-error', function (global, msg, detail) { notifyError(msg, detail); });
+
     Gio.DesktopAppInfo.set_desktop_env('GNOME');
 
     shellDBusService = new ShellDBus.GnomeShell();
@@ -99,8 +89,6 @@ function start() {
     // not loading any events until the user presses the clock
     global.launch_calendar_server();
 
-    Environment.init();
-
     // Ensure ShellWindowTracker and ShellAppUsage are initialized; this will
     // also initialize ShellAppSystem first.  ShellAppSystem
     // needs to load all the .desktop files, and ShellWindowTracker
@@ -133,6 +121,7 @@ function start() {
 
     // Set up stage hierarchy to group all UI actors under one container.
     uiGroup = new Clutter.Group();
+    St.set_ui_root(global.stage, uiGroup);
     global.window_group.reparent(uiGroup);
     global.overlay_group.reparent(uiGroup);
     global.stage.add_actor(uiGroup);
@@ -189,7 +178,7 @@ function start() {
     // Attempt to become a PolicyKit authentication agent
     PolkitAuthenticationAgent.init()
 
-    global.gdk_screen.connect('monitors-changed', _relayout);
+    global.screen.connect('monitors-changed', _relayout);
 
     ExtensionSystem.init();
     ExtensionSystem.loadExtensions();
@@ -216,7 +205,12 @@ function start() {
     }
 
     global.screen.connect('notify::n-workspaces', _nWorkspacesChanged);
-    Mainloop.idle_add(_nWorkspacesChanged);
+
+    global.screen.connect('window-entered-monitor', _windowEnteredMonitor);
+    global.screen.connect('window-left-monitor', _windowLeftMonitor);
+    global.screen.connect('restacked', _windowsRestacked);
+
+    _nWorkspacesChanged();
 }
 
 let _workspaces = [];
@@ -262,12 +256,32 @@ function _checkWorkspaces() {
         emptyWorkspaces.push(false);
     }
 
+    let activeWorkspaceIndex = global.screen.get_active_workspace_index();
+    let removingCurrentWorkspace = (emptyWorkspaces[activeWorkspaceIndex] &&
+                                    activeWorkspaceIndex < emptyWorkspaces.length - 1);
+    // Don't enter the overview when removing multiple empty workspaces at startup
+    let showOverview  = (removingCurrentWorkspace &&
+                         !emptyWorkspaces.every(function(x) { return x; }));
+
+    if (removingCurrentWorkspace) {
+        // "Merge" the empty workspace we are removing with the one at the end
+        wm.blockAnimations();
+    }
+
     // Delete other empty workspaces; do it from the end to avoid index changes
     for (i = emptyWorkspaces.length - 2; i >= 0; i--) {
         if (emptyWorkspaces[i])
             global.screen.remove_workspace(_workspaces[i], global.get_current_time());
     }
 
+    if (removingCurrentWorkspace) {
+        global.screen.get_workspace_by_index(global.screen.n_workspaces - 1).activate(global.get_current_time());
+        wm.unblockAnimations();
+
+        if (!overview.visible && showOverview)
+            overview.show();
+    }
+
     _checkWorkspacesId = 0;
     return false;
 }
@@ -283,6 +297,27 @@ function _windowRemoved(workspace, window) {
     });
 }
 
+function _windowLeftMonitor(metaScreen, monitorIndex, metaWin) {
+    // If the window left the primary monitor, that
+    // might make that workspace empty
+    if (monitorIndex == global.get_primary_monitor_index())
+        _queueCheckWorkspaces();
+}
+
+function _windowEnteredMonitor(metaScreen, monitorIndex, metaWin) {
+    // If the window entered the primary monitor, that
+    // might make that workspace non-empty
+    if (monitorIndex == global.get_primary_monitor_index())
+        _queueCheckWorkspaces();
+}
+
+function _windowsRestacked() {
+    // Figure out where the pointer is in case we lost track of
+    // it during a grab. (In particular, if a trayicon popup menu
+    // is dismissed, see if we need to close the message tray.)
+    global.sync_pointer();
+}
+
 function _queueCheckWorkspaces() {
     if (_checkWorkspacesId == 0)
         _checkWorkspacesId = Meta.later_add(Meta.LaterType.BEFORE_REDRAW, _checkWorkspaces);
@@ -366,15 +401,45 @@ function setThemeStylesheet(cssStylesheet)
  */
 function loadTheme() {
     let themeContext = St.ThemeContext.get_for_stage (global.stage);
+    let previousTheme = themeContext.get_theme();
 
     let cssStylesheet = _defaultCssStylesheet;
     if (_cssStylesheet != null)
         cssStylesheet = _cssStylesheet;
 
     let theme = new St.Theme ({ application_stylesheet: cssStylesheet });
+
+    if (previousTheme) {
+        let customStylesheets = previousTheme.get_custom_stylesheets();
+
+        for (let i = 0; i < customStylesheets.length; i++)
+            theme.load_stylesheet(customStylesheets[i]);
+    }
+
     themeContext.set_theme (theme);
 }
 
+/**
+ * notifyError:
+ * @msg: An error message
+ * @details: Additional information
+ *
+ * See shell_global_notify_problem().
+ */
+function notifyError(msg, details) {
+    // Also print to stderr so it's logged somewhere
+    if (details)
+        log("error: " + msg + ": " + details);
+    else
+        log("error: " + msg)
+
+    let source = new MessageTray.SystemNotificationSource();
+    messageTray.add(source);
+    let notification = new MessageTray.Notification(source, msg, details);
+    notification.setTransient(true);
+    source.notify(notification);
+}
+
 /**
  * _log:
  * @category: string message type ('info', 'error')
@@ -418,28 +483,67 @@ function _getAndClearErrorStack() {
 
 function _relayout() {
     let monitors = global.get_monitors();
-    if (monitors.length != hotCorners.length) {
-        // destroy old corners
-        for (let i = 0; i < hotCorners.length; i++)
-            hotCorners[i].destroy();
-        hotCorners = [];
-        for (let i = 0; i < monitors.length; i++)
-            hotCorners[i] = new Panel.HotCorner();
-    }
+    // destroy old corners
+    for (let i = 0; i < hotCorners.length; i++)
+        hotCorners[i].destroy();
+    hotCorners = [];
 
 
     let primary = global.get_primary_monitor();
     for (let i = 0; i < monitors.length; i++) {
         let monitor = monitors[i];
-        let corner = hotCorners[i];
         let isPrimary = (monitor.x == primary.x &&
                          monitor.y == primary.y &&
                          monitor.width == primary.width &&
                          monitor.height == primary.height);
+
+        let cornerX = monitor.x;
+        let cornerY = monitor.y;
         if (St.Widget.get_default_direction() == St.TextDirection.RTL)
-            corner.actor.set_position(monitor.x + monitor.width, monitor.y);
+            cornerX += monitor.width;
+
+
+        let haveTopLeftCorner = true;
+
+        /* Check if we have a top left (right for RTL) corner.
+         * I.e. if there is no monitor directly above or to the left(right) */
+        let besideX;
+        if (St.Widget.get_default_direction() == St.TextDirection.RTL)
+            besideX = monitor.x + 1;
         else
-            corner.actor.set_position(monitor.x, monitor.y);
+            besideX = cornerX - 1;
+        let besideY = cornerY;
+        let aboveX = cornerX;
+        let aboveY = cornerY - 1;
+
+        for (let j = 0; j < monitors.length; j++) {
+            if (i == j)
+                continue;
+            let otherMonitor = monitors[j];
+            if (besideX >= otherMonitor.x &&
+                besideX < otherMonitor.x + otherMonitor.width &&
+                besideY >= otherMonitor.y &&
+                besideY < otherMonitor.y + otherMonitor.height) {
+                haveTopLeftCorner = false;
+                break;
+            }
+            if (aboveX >= otherMonitor.x &&
+                aboveX < otherMonitor.x + otherMonitor.width &&
+                aboveY >= otherMonitor.y &&
+                aboveY < otherMonitor.y + otherMonitor.height) {
+                haveTopLeftCorner = false;
+                break;
+            }
+        }
+
+        /* We only want hot corners where there is a natural top-left
+         * corner, and on the primary monitor */
+        if (!isPrimary && !haveTopLeftCorner)
+            continue;
+
+        let corner = new Panel.HotCorner(isPrimary ? panel.button : null);
+        hotCorners.push(corner);
+        corner.actor.set_position(cornerX, cornerY);
         if (isPrimary)
             panel.setHotCorner(corner);
     }
