@@ -2,8 +2,6 @@
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 const Gtk = imports.gi.Gtk;
 const Meta = imports.gi.Meta;
 const St = imports.gi.St;
@@ -30,6 +28,7 @@ SearchResult.prototype = {
                                      x_align: St.Align.START,
                                      y_fill: true });
         this.actor._delegate = this;
+        this._dragActorSource = null;
 
         let content = provider.createResultActor(metaInfo, terms);
         if (content == null) {
@@ -39,6 +38,11 @@ SearchResult.prototype = {
             let icon = new IconGrid.BaseIcon(this.metaInfo['name'],
                                              { createIcon: this.metaInfo['createIcon'] });
             content.set_child(icon.actor);
+            this._dragActorSource = icon.icon;
+            this.actor.label_actor = icon.label;
+        } else {
+            if (content._delegate && content._delegate.getDragActorSource)
+                this._dragActorSource = content._delegate.getDragActorSource();
         }
         this._content = content;
         this.actor.set_child(content);
@@ -77,6 +81,8 @@ SearchResult.prototype = {
     },
 
     getDragActorSource: function() {
+        if (this._dragActorSource)
+            return this._dragActorSource;
         // not exactly right, but alignment problems are hard to notice
         return this._content;
     },
@@ -193,7 +199,7 @@ SearchResults.prototype = {
 
         let scrollView = new St.ScrollView({ x_fill: true,
                                              y_fill: false,
-                                             vfade: true });
+                                             style_class: 'vfade' });
         scrollView.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
         scrollView.add_actor(this._content);
 
@@ -259,6 +265,7 @@ SearchResults.prototype = {
         let title = new St.Label({ text: provider.name,
                                    style_class: 'dash-search-button-label' });
 
+        button.label_actor = title;
         bin.set_child(title);
         button.set_child(bin);
         provider.actor = button;
