@@ -9,9 +9,6 @@ const Signals = imports.signals;
 const St = imports.gi.St;
 const Tpl = imports.gi.TelepathyLogger;
 const Tp = imports.gi.TelepathyGLib;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
-const C_ = Gettext.pgettext;
 
 const History = imports.misc.history;
 const Main = imports.ui.main;
@@ -80,19 +77,20 @@ Client.prototype = {
         // The second argument, recover, means _observeChannels will be run
         // for any existing channel as well.
         let dbus = Tp.DBusDaemon.dup();
-        this._observer = Tp.SimpleObserver.new(dbus, true, 'GnomeShell', true,
-                                              Lang.bind(this, this._observeChannels));
-
-        // We only care about single-user text-based chats
-        let props = {};
-        props[Tp.PROP_CHANNEL_CHANNEL_TYPE] = Tp.IFACE_CHANNEL_TYPE_TEXT;
-        props[Tp.PROP_CHANNEL_TARGET_HANDLE_TYPE] = Tp.HandleType.CONTACT;
-        this._observer.add_observer_filter(props);
+        this._tpClient = new Shell.TpClient({ 'dbus_daemon': dbus,
+                                              'name': 'GnomeShell',
+                                              'uniquify-name': true })
+        this._tpClient.set_observe_channels_func(
+            Lang.bind(this, this._observeChannels));
+        this._tpClient.set_approve_channels_func(
+            Lang.bind(this, this._approveChannels));
+        this._tpClient.set_handle_channels_func(
+            Lang.bind(this, this._handleChannels));
 
         try {
-            this._observer.register();
+            this._tpClient.register();
         } catch (e) {
-            throw new Error('Couldn\'t register SimpleObserver. Error: \n' + e);
+            throw new Error('Couldn\'t register Telepathy client. Error: \n' + e);
         }
     },
 
@@ -105,7 +103,7 @@ Client.prototype = {
             this._finishObserveChannels(account, conn, channels, context);
         } else {
             Shell.get_self_contact_features(conn,
-                                            contactFeatures.length, contactFeatures,
+                                            contactFeatures,
                                             Lang.bind(this, function() {
                                                 this._finishObserveChannels(account, conn, channels, context);
                                             }));
@@ -125,8 +123,8 @@ Client.prototype = {
                continue;
 
             /* Request a TpContact */
-            Shell.get_tp_contacts(conn, 1, [targetHandle],
-                    contactFeatures.length, contactFeatures,
+            Shell.get_tp_contacts(conn, [targetHandle],
+                    contactFeatures,
                     Lang.bind(this,  function (connection, contacts, failed) {
                         if (contacts.length < 1)
                             return;
@@ -143,30 +141,82 @@ Client.prototype = {
         if (this._sources[channel.get_object_path()])
             return;
 
-        let source = new Source(account, conn, channel, contact);
+        let source = new Source(account, conn, channel, contact, this._tpClient);
 
         this._sources[channel.get_object_path()] = source;
         source.connect('destroy', Lang.bind(this,
                        function() {
+                           if (this._tpClient.is_handling_channel(channel)) {
+                               // The chat box has been destroyed so it can't
+                               // handle the channel any more.
+                               channel.close_async(function(src, result) {
+                                   channel.close_finish(result);
+                               });
+                           }
+
                            delete this._sources[channel.get_object_path()];
                        }));
+    },
+
+    _handlingChannels: function(account, conn, channels) {
+        let len = channels.length;
+        for (let i = 0; i < len; i++) {
+            let channel = channels[i];
+
+            // We can only handle text channel, so close any other channel
+            if (!(channel instanceof Tp.TextChannel)) {
+                channel.close_async(null);
+                continue;
+            }
+
+            if (this._tpClient.is_handling_channel(channel)) {
+                // We are already handling the channel, display the source
+                let source = this._sources[channel.get_object_path()];
+                if (source)
+                    source.notify();
+            }
+        }
+    },
+
+    _approveChannels: function(approver, account, conn, channels,
+                               dispatchOp, context) {
+        // Approve the channels right away as we are going to handle it
+        dispatchOp.claim_with_async(this._tpClient,
+                                    Lang.bind (this, function(dispatchOp, result) {
+            try {
+                dispatchOp.claim_with_finish(result);
+                this._handlingChannels(account, conn, channels);
+            } catch (err) {
+                global.logError('Failed to Claim channel: ' + err);
+            }}));
+
+        context.accept();
+    },
+
+    _handleChannels: function(handler, account, conn, channels,
+                              requests, user_action_time, context) {
+        this._handlingChannels(account, conn, channels);
+        context.accept();
     }
 };
 
-function Source(account, conn, channel, contact) {
-    this._init(account, conn, channel, contact);
+function Source(account, conn, channel, contact, client) {
+    this._init(account, conn, channel, contact, client);
 }
 
 Source.prototype = {
     __proto__:  MessageTray.Source.prototype,
 
-    _init: function(account, conn, channel, contact) {
+    _init: function(account, conn, channel, contact, client) {
         MessageTray.Source.prototype._init.call(this, contact.get_alias());
 
         this.isChat = true;
 
         this._account = account;
         this._contact = contact;
+        this._client = client;
+
+        this._pendingMessages = [];
 
         this._conn = conn;
         this._channel = channel;
@@ -175,10 +225,21 @@ Source.prototype = {
         this._notification = new Notification(this);
         this._notification.setUrgency(MessageTray.Urgency.HIGH);
 
+        // We ack messages when the message box is collapsed if user has
+        // interacted with it before and so read the messages:
+        // - user clicked on it the tray
+        // - user expanded the notification by hovering over the toaster notification
+        this._shouldAck = false;
+
+        this.connect('summary-item-clicked', Lang.bind(this, this._summaryItemClicked));
+        this._notification.connect('expanded', Lang.bind(this, this._notificationExpanded));
+        this._notification.connect('collapsed', Lang.bind(this, this._notificationCollapsed));
+
         this._presence = contact.get_presence_type();
 
         this._sentId = this._channel.connect('message-sent', Lang.bind(this, this._messageSent));
         this._receivedId = this._channel.connect('message-received', Lang.bind(this, this._messageReceived));
+        this._pendingId = this._channel.connect('pending-message-removed', Lang.bind(this, this._pendingRemoved));
 
         this._setSummaryIcon(this.createNotificationIcon());
 
@@ -195,7 +256,7 @@ Source.prototype = {
 
     _updateAlias: function() {
         let oldAlias = this.title;
-        this.title = this._contact.get_alias();
+        this.setTitle(this._contact.get_alias());
         this._notification.appendAliasChange(oldAlias, this.title);
         this.pushNotification(this._notification);
     },
@@ -224,13 +285,17 @@ Source.prototype = {
     },
 
     open: function(notification) {
-        let props = {};
-        props[Tp.PROP_CHANNEL_CHANNEL_TYPE] = Tp.IFACE_CHANNEL_TYPE_TEXT;
-        [props[Tp.PROP_CHANNEL_TARGET_HANDLE], props[Tp.PROP_CHANNEL_TARGET_HANDLE_TYPE]] = this._channel.get_handle();
+          if (this._client.is_handling_channel(this._channel)) {
+              // We are handling the channel, try to pass it to Empathy
+              this._client.delegate_channels_async([this._channel], global.get_current_time(), "", null);
+          }
+          else {
+              // We are not the handler, just ask to present the channel
+              let dbus = Tp.DBusDaemon.dup();
+              let cd = Tp.ChannelDispatcher.new(dbus);
 
-        let req = Tp.AccountChannelRequest.new(this._account, props, global.get_current_time());
-
-        req.ensure_channel_async('', null, null);
+              cd.present_channel_async(this._channel, global.get_current_time(), null);
+          }
     },
 
     _getLogMessages: function() {
@@ -248,7 +313,18 @@ Source.prototype = {
         let logMessages = events.map(makeMessageFromTplEvent);
 
         let pendingTpMessages = this._channel.get_pending_messages();
-        let pendingMessages = pendingTpMessages.map(function (tpMessage) { return makeMessageFromTpMessage(tpMessage, NotificationDirection.RECEIVED); });
+        let pendingMessages = [];
+
+        for (let i = 0; i < pendingTpMessages.length; i++) {
+            let message = pendingTpMessages[i];
+
+            if (message.get_message_type() == Tp.ChannelTextMessageType.DELIVERY_REPORT)
+                continue;
+
+            pendingMessages.push(makeMessageFromTpMessage(message, NotificationDirection.RECEIVED));
+
+            this._pendingMessages.push(message);
+        }
 
         let showTimestamp = false;
 
@@ -284,6 +360,7 @@ Source.prototype = {
     _channelClosed: function() {
         this._channel.disconnect(this._closedId);
         this._channel.disconnect(this._receivedId);
+        this._channel.disconnect(this._pendingId);
         this._channel.disconnect(this._sentId);
 
         this._contact.disconnect(this._notifyAliasId);
@@ -294,6 +371,11 @@ Source.prototype = {
     },
 
     _messageReceived: function(channel, message) {
+        if (message.get_message_type() == Tp.ChannelTextMessageType.DELIVERY_REPORT)
+            return;
+
+        this._pendingMessages.push(message);
+
         message = makeMessageFromTpMessage(message, NotificationDirection.RECEIVED);
         this._notification.appendMessage(message);
         this.notify();
@@ -320,10 +402,12 @@ Source.prototype = {
         }
 
         let msg = Tp.ClientMessage.new_text(type, text);
-        this._channel.send_message_async(msg, 0, null);
+        this._channel.send_message_async(msg, 0, Lang.bind(this, function (src, result) {
+            this._channel.send_message_finish(result); 
+        }));
     },
 
-    _presenceChanged: function (contact, presence, type, status, message) {
+    _presenceChanged: function (contact, presence, status, message) {
         let msg, shouldNotify, title;
 
         if (this._presence == presence)
@@ -356,6 +440,43 @@ Source.prototype = {
         this._notification.appendPresence(msg, shouldNotify);
         if (shouldNotify)
             this.notify();
+    },
+
+    _pendingRemoved: function(channel, message) {
+        let idx = this._pendingMessages.indexOf(message);
+
+        if (idx >= 0)
+            this._pendingMessages.splice(idx, 1);
+        else
+            throw new Error('Message not in our pending list: ' + message);
+    },
+
+    _ackMessages: function() {
+        if (this._pendingMessages.length == 0)
+            return;
+
+        // Don't clear our messages here, tp-glib will send a
+        // 'pending-message-removed' for each one.
+        this._channel.ack_messages_async(this._pendingMessages, Lang.bind(this, function(src, result) {
+            this._channel.ack_messages_finish(result);}));
+    },
+
+    _summaryItemClicked: function(source, button) {
+        if (button != 1)
+            return;
+
+        this._shouldAck = true;
+    },
+
+    _notificationExpanded: function() {
+        this._shouldAck = true;
+    },
+
+    _notificationCollapsed: function() {
+        if (this._shouldAck)
+            this._ackMessages();
+
+        this._shouldAck = false;
     }
 };
 
@@ -424,13 +545,35 @@ Notification.prototype = {
         this._append(messageBody, styles, message.timestamp, noTimestamp);
     },
 
+    _filterMessages: function() {
+        if (this._history.length < 1)
+            return;
+
+        let lastMessageTime = this._history[0].time;
+        let currentTime = (Date.now() / 1000);
+
+        // Keep the scrollback from growing too long. If the most
+        // recent message (before the one we just added) is within
+        // SCROLLBACK_RECENT_TIME, we will keep
+        // SCROLLBACK_RECENT_LENGTH previous messages. Otherwise
+        // we'll keep SCROLLBACK_IDLE_LENGTH messages.
+
+        let maxLength = (lastMessageTime < currentTime - SCROLLBACK_RECENT_TIME) ?
+            SCROLLBACK_IDLE_LENGTH : SCROLLBACK_RECENT_LENGTH;
+
+        let filteredHistory = this._history.filter(function(item) { return item.realMessage });
+        if (filteredHistory.length > maxLength) {
+            let lastMessageToKeep = filteredHistory[maxLength];
+            let expired = this._history.splice(this._history.indexOf(lastMessageToKeep));
+            for (let i = 0; i < expired.length; i++)
+                expired[i].actor.destroy();
+        }
+    },
+
     _append: function(text, styles, timestamp, noTimestamp) {
         let currentTime = (Date.now() / 1000);
         if (!timestamp)
             timestamp = currentTime;
-        let lastMessageTime = -1;
-        if (this._history.length > 0)
-            lastMessageTime = this._history[0].time;
 
         // Reset the old message timeout
         if (this._timestampTimeoutId)
@@ -453,23 +596,7 @@ Notification.prototype = {
                     Lang.bind(this, this.appendTimestamp));
         }
 
-        if (this._history.length > 1) {
-            // Keep the scrollback from growing too long. If the most
-            // recent message (before the one we just added) is within
-            // SCROLLBACK_RECENT_TIME, we will keep
-            // SCROLLBACK_RECENT_LENGTH previous messages. Otherwise
-            // we'll keep SCROLLBACK_IDLE_LENGTH messages.
-
-            let maxLength = (lastMessageTime < currentTime - SCROLLBACK_RECENT_TIME) ?
-                SCROLLBACK_IDLE_LENGTH : SCROLLBACK_RECENT_LENGTH;
-            let filteredHistory = this._history.filter(function(item) { return item.realMessage });
-            if (filteredHistory.length > maxLength) {
-                let lastMessageToKeep = filteredHistory[maxLength];
-                let expired = this._history.splice(this._history.indexOf(lastMessageToKeep));
-                for (let i = 0; i < expired.length; i++)
-                    expired[i].actor.destroy();
-            }
-        }
+        this._filterMessages();
     },
 
     _formatTimestamp: function(date) {
@@ -511,6 +638,9 @@ Notification.prototype = {
         this._history.unshift({ actor: timeLabel, time: lastMessageTime, realMessage: false });
 
         this._timestampTimeoutId = 0;
+
+        this._filterMessages();
+
         return false;
     },
 
@@ -522,21 +652,23 @@ Notification.prototype = {
         let label = this.addBody(text, true);
         label.add_style_class_name('chat-meta-message');
         this._history.unshift({ actor: label, time: (Date.now() / 1000), realMessage: false});
+
+        this._filterMessages();
     },
 
     appendAliasChange: function(oldAlias, newAlias) {
-        // FIXME: uncomment this after 3.0 string freeze ends
+        oldAlias = GLib.markup_escape_text(oldAlias, -1);
+        newAlias = GLib.markup_escape_text(newAlias, -1);
 
-        // oldAlias = GLib.markup_escape_text(oldAlias, -1);
-        // newAlias = GLib.markup_escape_text(newAlias, -1);
+        /* Translators: this is the other person changing their old IM name to their new
+           IM name. */
+        let message = '<i>' + _("%s is now known as %s").format(oldAlias, newAlias) + '</i>';
+        let label = this.addBody(message, true);
+        label.add_style_class_name('chat-meta-message');
+        this._history.unshift({ actor: label, time: (Date.now() / 1000), realMessage: false });
+        this.update(newAlias, null, { customContent: true });
 
-        // /* Translators: this is the other person changing their old IM name to their new
-        //    IM name. */
-        // let message = '<i>' + _("%s is now known as %s").format(oldAlias, newAlias) + '</i>';
-        // let label = this.addBody(message, true);
-        // label.add_style_class_name('chat-meta-message');
-        // this._history.unshift({ actor: label, time: (Date.now() / 1000), realMessage: false });
-        // this.update(newAlias, null, { customContent: true });
+        this._filterMessages();
     },
 
     _onEntryActivated: function() {
