@@ -6,6 +6,7 @@ const Gio = imports.gi.Gio;
 const Lang = imports.lang;
 const Meta = imports.gi.Meta;
 const St = imports.gi.St;
+const Shell = imports.gi.Shell;
 
 const AltTab = imports.ui.altTab;
 const WorkspaceSwitcherPopup = imports.ui.workspaceSwitcherPopup;
@@ -22,8 +23,7 @@ function getDimShader() {
     if (dimShader === null)
         return null;
     if (!dimShader) {
-        let [success, source, length] = GLib.file_get_contents(global.datadir +
-                                                               '/shaders/dim-window.glsl');
+        let source = Shell.get_file_contents_utf8_sync(global.datadir + '/shaders/dim-window.glsl');
         try {
             let shader = new Clutter.Shader();
             shader.set_fragment_source(source, -1);
