@@ -7,8 +7,6 @@ const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
 const Signals = imports.signals;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const DND = imports.ui.dnd;
 const Main = imports.ui.main;
@@ -25,14 +23,13 @@ const MAX_WORKSPACES = 16;
 const CONTROLS_POP_IN_TIME = 0.1;
 
 
-function WorkspacesView(width, height, x, y, workspaces) {
-    this._init(width, height, x, y, workspaces);
+function WorkspacesView(workspaces) {
+    this._init(workspaces);
 }
 
 WorkspacesView.prototype = {
-    _init: function(width, height, x, y, workspaces) {
+    _init: function(workspaces) {
         this.actor = new St.Group({ style_class: 'workspaces-view' });
-        this.actor.set_clip(x, y, width, height);
 
         // The actor itself isn't a drop target, so we don't want to pick on its area
         this.actor.set_size(0, 0);
@@ -43,20 +40,21 @@ WorkspacesView.prototype = {
             function() {
                 let node = this.actor.get_theme_node();
                 this._spacing = node.get_length('spacing');
-                this._computeWorkspacePositions();
+                this._updateWorkspaceActors(false);
             }));
         this.actor.connect('notify::mapped',
                            Lang.bind(this, this._onMappedChanged));
 
-        this._width = width;
-        this._height = height;
-        this._x = x;
-        this._y = y;
-        this._zoomScale = 1.0;
+        this._width = 0;
+        this._height = 0;
+        this._x = 0;
+        this._y = 0;
+        this._clipX = 0;
+        this._clipY = 0;
+        this._clipWidth = 0;
+        this._clipHeight = 0;
+        this._workspaceRatioSpacing = 0;
         this._spacing = 0;
-        this._activeWorkspaceX = 0; // x offset of active ws while dragging
-        this._activeWorkspaceY = 0; // y offset of active ws while dragging
-        this._lostWorkspaces = [];
         this._animating = false; // tweening
         this._scrolling = false; // swipe-scrolling
         this._animatingScroll = false; // programatically updating the adjustment
@@ -71,6 +69,18 @@ WorkspacesView.prototype = {
             this._workspaces[w].actor.reparent(this.actor);
         this._workspaces[activeWorkspaceIndex].actor.raise_top();
 
+        this._extraWorkspaces = [];
+        let monitors = global.get_monitors();
+        let m = 0;
+        for (let i = 0; i < monitors.length; i++) {
+            if (i == global.get_primary_monitor_index())
+                continue;
+            let ws = new Workspace.Workspace(null, i);
+            this._extraWorkspaces[m++] = ws;
+            ws.setGeometry(monitors[i].x, monitors[i].y, monitors[i].width, monitors[i].height);
+            global.overlay_group.add_actor(ws.actor);
+        }
+
         // Position/scale the desktop windows and their children after the
         // workspaces have been created. This cannot be done first because
         // window movement depends on the Workspaces object being accessible
@@ -80,6 +90,14 @@ WorkspacesView.prototype = {
                                  Lang.bind(this, function() {
                 for (let w = 0; w < this._workspaces.length; w++)
                     this._workspaces[w].zoomToOverview();
+                for (let w = 0; w < this._extraWorkspaces.length; w++)
+                    this._extraWorkspaces[w].zoomToOverview();
+        }));
+        this._overviewShownId =
+            Main.overview.connect('shown',
+                                 Lang.bind(this, function() {
+                this.actor.set_clip(this._clipX, this._clipY,
+                                    this._clipWidth, this._clipHeight);
         }));
 
         this._scrollAdjustment = new St.Adjustment({ value: activeWorkspaceIndex,
@@ -109,30 +127,25 @@ WorkspacesView.prototype = {
         this._swipeScrollEndId = 0;
     },
 
-    setZoomScale: function(zoomScale) {
-        if (zoomScale == this._zoomScale)
-            return;
+    setGeometry: function(x, y, width, height, spacing) {
+      if (this._x == x && this._y == y &&
+          this._width == width && this._height == height)
+          return;
+        this._width = width;
+        this._height = height;
+        this._x = x;
+        this._y = y;
+        this._workspaceRatioSpacing = spacing;
 
-        this._zoomScale = zoomScale;
-        if (this._zoomOut) {
-            // If we are already zoomed out, then we have to reposition.
-            // Note that when shown initially zoomOut is false, so we
-            // won't trigger this.
+        for (let i = 0; i < this._workspaces.length; i++)
+            this._workspaces[i].setGeometry(x, y, width, height);
+    },
 
-            // setZoomScale can be invoked when the workspaces view is
-            // reallocated. Since we just want to animate things to the
-            // new position it seems OK to call updateWorkspaceActors
-            // immediately - adding a tween doesn't immediately cause
-            // a new allocation. But hide/show of the window overlays we
-            // do around animation does, so we need to do it later.
-            // This can be removed when we fix things to not hide/show
-            // the window overlay.
-            Meta.later_add(Meta.LaterType.BEFORE_REDRAW,
-                           Lang.bind(this, function() {
-                                         this._computeWorkspacePositions();
-                                         this._updateWorkspaceActors(true);
-                                     }));
-        }
+    setClipRect: function(x, y, width, height) {
+        this._clipX = x;
+        this._clipY = y;
+        this._clipWidth = width;
+        this._clipHeight = height;
     },
 
     _lookupWorkspaceForMetaWindow: function (metaWindow) {
@@ -148,107 +161,51 @@ WorkspacesView.prototype = {
         return this._workspaces[active];
     },
 
+    getWorkspaceByIndex: function(index) {
+        return this._workspaces[index];
+    },
+
     hide: function() {
         let activeWorkspaceIndex = global.screen.get_active_workspace_index();
         let activeWorkspace = this._workspaces[activeWorkspaceIndex];
 
         activeWorkspace.actor.raise_top();
 
+       this.actor.remove_clip(this._x, this._y, this._width, this._height);
+
         for (let w = 0; w < this._workspaces.length; w++)
             this._workspaces[w].zoomFromOverview();
+        for (let w = 0; w < this._extraWorkspaces.length; w++)
+            this._extraWorkspaces[w].zoomFromOverview();
     },
 
     destroy: function() {
         this.actor.destroy();
     },
 
-    getScale: function() {
-        return this._workspaces[0].scale;
-    },
-
     syncStacking: function(stackIndices) {
         for (let i = 0; i < this._workspaces.length; i++)
             this._workspaces[i].syncStacking(stackIndices);
+        for (let i = 0; i < this._extraWorkspaces.length; i++)
+            this._extraWorkspaces[i].syncStacking(stackIndices);
     },
 
-    // Get the grid position of the active workspace.
-    getActiveWorkspacePosition: function() {
-        let activeWorkspaceIndex = global.screen.get_active_workspace_index();
-        let activeWorkspace = this._workspaces[activeWorkspaceIndex];
-
-        return [activeWorkspace.x, activeWorkspace.y];
-    },
-
-    zoomOut: function() {
-        if (this._zoomOut)
-            return;
-
-        this._zoomOut = true;
-        this._computeWorkspacePositions();
-        this._updateWorkspaceActors(true);
-    },
-
-    zoomIn: function() {
-        if (!this._zoomOut)
-            return;
-
-        this._zoomOut = false;
-        this._computeWorkspacePositions();
-        this._updateWorkspaceActors(true);
-    },
-
-    // Compute the position, scale and opacity of the workspaces, but don't
-    // actually change the actors to match
-    _computeWorkspacePositions: function() {
-        let active = global.screen.get_active_workspace_index();
-        let zoomScale = this._zoomOut ? this._zoomScale : 1;
-        let scale = zoomScale * this._width / global.screen_width;
-
-        let _width = this._workspaces[0].actor.width * scale;
-        let _height = this._workspaces[0].actor.height * scale;
-
-        this._activeWorkspaceX = (this._width - _width) / 2;
-        this._activeWorkspaceY = (this._height - _height) / 2;
-
-        for (let w = 0; w < this._workspaces.length; w++) {
-            let workspace = this._workspaces[w];
-
-            workspace.opacity = (this._inDrag && w != active) ? 200 : 255;
-
-            workspace.scale = scale;
-            workspace.x = this._x + this._activeWorkspaceX;
-
-            // We adjust the center because the zoomScale is to leave space for
-            // the expanded workspace control so we want to zoom to either the
-            // left part of the area or the right part of the area
-            let offset = 0.5 * (1 - this._zoomScale) * this._width;
-            let rtl = (St.Widget.get_default_direction () == St.TextDirection.RTL);
-            if (this._zoomOut)
-                workspace.x += rtl ? offset : - offset;
-
-            // We divide by zoomScale so that adjacent workspaces are always offscreen
-            // except when we are switching between workspaces
-            workspace.y = this._y + this._activeWorkspaceY
-                                  + (w - active) * (_height + this._spacing) / zoomScale;
-        }
+    updateWindowPositions: function() {
+        for (let w = 0; w < this._workspaces.length; w++)
+            this._workspaces[w].positionWindows(Workspace.WindowPositionFlags.ANIMATE);
     },
 
     _scrollToActive: function(showAnimation) {
         let active = global.screen.get_active_workspace_index();
 
-        this._computeWorkspacePositions();
         this._updateWorkspaceActors(showAnimation);
         this._updateScrollAdjustment(active, showAnimation);
     },
 
-    // Update workspace actors parameters to the values calculated in
-    // _computeWorkspacePositions()
+    // Update workspace actors parameters
     // @showAnimation: iff %true, transition between states
     _updateWorkspaceActors: function(showAnimation) {
         let active = global.screen.get_active_workspace_index();
-        let targetWorkspaceNewY = this._y + this._activeWorkspaceY;
-        let targetWorkspaceCurrentY = this._workspaces[active].y;
-        let dy = targetWorkspaceNewY - targetWorkspaceCurrentY;
 
         this._animating = showAnimation;
 
@@ -257,14 +214,10 @@ WorkspacesView.prototype = {
 
             Tweener.removeTweens(workspace.actor);
 
-            workspace.y += dy;
+            let y = (w - active) * (this._height + this._spacing + this._workspaceRatioSpacing);
 
             if (showAnimation) {
-                let params = { x: workspace.x,
-                               y: workspace.y,
-                               scale_x: workspace.scale,
-                               scale_y: workspace.scale,
-                               opacity: workspace.opacity,
+                let params = { y: y,
                                time: WORKSPACE_SWITCH_TIME,
                                transition: 'easeOutQuad'
                              };
@@ -281,35 +234,11 @@ WorkspacesView.prototype = {
                 }
                 Tweener.addTween(workspace.actor, params);
             } else {
-                workspace.actor.set_scale(workspace.scale, workspace.scale);
-                workspace.actor.set_position(workspace.x, workspace.y);
-                workspace.actor.opacity = workspace.opacity;
+                workspace.actor.set_position(0, y);
                 if (w == 0)
                     this._updateVisibility();
             }
         }
-
-        for (let l = 0; l < this._lostWorkspaces.length; l++) {
-            let workspace = this._lostWorkspaces[l];
-
-            Tweener.removeTweens(workspace.actor);
-
-            workspace.y += dy;
-            workspace.actor.show();
-            workspace.hideWindowsOverlays();
-
-            if (showAnimation) {
-                Tweener.addTween(workspace.actor,
-                                 { y: workspace.x,
-                                   time: WORKSPACE_SWITCH_TIME,
-                                   transition: 'easeOutQuad',
-                                   onComplete: Lang.bind(this,
-                                                         this._cleanWorkspaces)
-                                 });
-            } else {
-                this._cleanWorkspaces();
-            }
-        }
     },
 
     _updateVisibility: function() {
@@ -330,18 +259,6 @@ WorkspacesView.prototype = {
         }
     },
 
-    _cleanWorkspaces: function() {
-        if (this._lostWorkspaces.length == 0)
-            return;
-
-        for (let l = 0; l < this._lostWorkspaces.length; l++)
-            this._lostWorkspaces[l].destroy();
-        this._lostWorkspaces = [];
-
-        this._computeWorkspacePositions();
-        this._updateWorkspaceActors(false);
-    },
-
     _updateScrollAdjustment: function(index, showAnimation) {
         if (this._scrolling)
             return;
@@ -364,12 +281,9 @@ WorkspacesView.prototype = {
         }
     },
 
-    updateWorkspaces: function(oldNumWorkspaces, newNumWorkspaces, lostWorkspaces) {
+    updateWorkspaces: function(oldNumWorkspaces, newNumWorkspaces) {
         let active = global.screen.get_active_workspace_index();
 
-        for (let l = 0; l < lostWorkspaces.length; l++)
-            lostWorkspaces[l].disconnectAll();
-
         Tweener.addTween(this._scrollAdjustment,
                          { upper: newNumWorkspaces,
                            time: WORKSPACE_SWITCH_TIME,
@@ -377,13 +291,13 @@ WorkspacesView.prototype = {
                          });
 
         if (newNumWorkspaces > oldNumWorkspaces) {
-            for (let w = oldNumWorkspaces; w < newNumWorkspaces; w++)
+            for (let w = oldNumWorkspaces; w < newNumWorkspaces; w++) {
+                this._workspaces[w].setGeometry(this._x, this._y,
+                                                this._width, this._height);
                 this.actor.add_actor(this._workspaces[w].actor);
+            }
 
-            this._computeWorkspacePositions();
             this._updateWorkspaceActors(false);
-        } else {
-            this._lostWorkspaces = lostWorkspaces;
         }
 
         this._scrollToActive(true);
@@ -397,10 +311,16 @@ WorkspacesView.prototype = {
     },
 
     _onDestroy: function() {
+        for (let i = 0; i < this._extraWorkspaces.length; i++)
+            this._extraWorkspaces[i].destroy();
         this._scrollAdjustment.run_dispose();
         Main.overview.disconnect(this._overviewShowingId);
+        Main.overview.disconnect(this._overviewShownId);
         global.window_manager.disconnect(this._switchWorkspaceNotifyId);
 
+        if (this._inDrag)
+            this._dragEnd();
+
         if (this._timeoutId) {
             Mainloop.source_remove(this._timeoutId);
             this._timeoutId = 0;
@@ -443,6 +363,7 @@ WorkspacesView.prototype = {
             return;
 
         this._inDrag = true;
+        this._firstDragMotion = true;
 
         this._dragMonitor = {
             dragMotion: Lang.bind(this, this._onDragMotion)
@@ -454,6 +375,14 @@ WorkspacesView.prototype = {
         if (Main.overview.animationInProgress)
              return DND.DragMotionResult.CONTINUE;
 
+        if (this._firstDragMotion) {
+            this._firstDragMotion = false;
+            for (let i = 0; i < this._workspaces.length; i++)
+                this._workspaces[i].setReservedSlot(dragEvent.dragActor._delegate);
+            for (let i = 0; i < this._extraWorkspaces.length; i++)
+                this._extraWorkspaces[i].setReservedSlot(dragEvent.dragActor._delegate);
+        }
+
         let primary = global.get_primary_monitor();
 
         let activeWorkspaceIndex = global.screen.get_active_workspace_index();
@@ -467,7 +396,6 @@ WorkspacesView.prototype = {
         let switchTop = (dragEvent.y <= topEdge && topWorkspace);
         if (switchTop && this._dragOverLastY != topEdge) {
             topWorkspace.metaWorkspace.activate(global.get_current_time());
-            topWorkspace.setReservedSlot(dragEvent.dragActor._delegate);
             this._dragOverLastY = topEdge;
 
             return DND.DragMotionResult.CONTINUE;
@@ -476,7 +404,6 @@ WorkspacesView.prototype = {
         let switchBottom = (dragEvent.y >= bottomEdge && bottomWorkspace);
         if (switchBottom && this._dragOverLastY != bottomEdge) {
             bottomWorkspace.metaWorkspace.activate(global.get_current_time());
-            bottomWorkspace.setReservedSlot(dragEvent.dragActor._delegate);
             this._dragOverLastY = bottomEdge;
 
             return DND.DragMotionResult.CONTINUE;
@@ -488,20 +415,14 @@ WorkspacesView.prototype = {
         if (topWorkspace) {
             if (topWorkspace.actor.contains(dragEvent.targetActor)) {
                 hoverWorkspace = topWorkspace;
-                topWorkspace.opacity = topWorkspace.actor.opacity = 255;
                 result = topWorkspace.handleDragOver(dragEvent.source, dragEvent.dragActor);
-            } else {
-                topWorkspace.opacity = topWorkspace.actor.opacity = 200;
             }
         }
 
         if (bottomWorkspace) {
             if (bottomWorkspace.actor.contains(dragEvent.targetActor)) {
                 hoverWorkspace = bottomWorkspace;
-                bottomWorkspace.opacity = bottomWorkspace.actor.opacity = 255;
                 result = bottomWorkspace.handleDragOver(dragEvent.source, dragEvent.dragActor);
-            } else {
-                bottomWorkspace.opacity = bottomWorkspace.actor.opacity = 200;
             }
         }
 
@@ -511,7 +432,6 @@ WorkspacesView.prototype = {
                 this._timeoutId = Mainloop.timeout_add_seconds(1,
                     Lang.bind(this, function() {
                        hoverWorkspace.metaWorkspace.activate(global.get_current_time());
-                       hoverWorkspace.setReservedSlot(dragEvent.dragActor._delegate);
                        return false;
                     }));
         } else {
@@ -529,11 +449,13 @@ WorkspacesView.prototype = {
             Mainloop.source_remove(this._timeoutId);
             this._timeoutId = 0;
         }
-        DND.removeMonitor(this._dragMonitor);
+        DND.removeDragMonitor(this._dragMonitor);
         this._inDrag = false;
 
         for (let i = 0; i < this._workspaces.length; i++)
             this._workspaces[i].setReservedSlot(null);
+        for (let i = 0; i < this._extraWorkspaces.length; i++)
+            this._extraWorkspaces[i].setReservedSlot(null);
     },
 
     _swipeScrollBegin: function() {
@@ -557,12 +479,6 @@ WorkspacesView.prototype = {
                 Main.overview.hide();
         }
 
-        if (result == Overview.SwipeScrollResult.SWIPE)
-            // The active workspace has changed; while swipe-scrolling
-            // has already taken care of the positioning, the cached
-            // positions need to be updated
-            this._computeWorkspacePositions();
-
         // Make sure title captions etc are shown as necessary
         this._updateVisibility();
     },
@@ -590,7 +506,7 @@ WorkspacesView.prototype = {
             return;
 
         let currentY = firstWorkspaceY;
-        let newY = this._y - adj.value / (adj.upper - 1) * workspacesHeight;
+        let newY =  - adj.value / (adj.upper - 1) * workspacesHeight;
 
         let dy = newY - currentY;
 
@@ -618,6 +534,7 @@ WorkspacesDisplay.prototype = {
         this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
         this.actor.connect('allocate', Lang.bind(this, this._allocate));
+        this.actor.set_clip_to_allocation(true);
 
         let controls = new St.Bin({ style_class: 'workspace-controls',
                                     request_mode: Clutter.RequestMode.WIDTH_FOR_HEIGHT,
@@ -633,6 +550,8 @@ WorkspacesDisplay.prototype = {
         controls.connect('scroll-event',
                          Lang.bind(this, this._onScrollEvent));
 
+        this._monitorIndex = global.get_primary_monitor_index();
+        this._monitor = global.get_monitors()[this._monitorIndex];
 
         this._thumbnailsBox = new WorkspaceThumbnail.ThumbnailsBox();
         controls.add_actor(this._thumbnailsBox.actor);
@@ -640,69 +559,55 @@ WorkspacesDisplay.prototype = {
         this.workspacesView = null;
 
         this._inDrag = false;
+        this._cancelledDrag = false;
+
+        this._alwaysZoomOut = false;
         this._zoomOut = false;
         this._zoomFraction = 0;
 
-        this._nWorkspacesNotifyId = 0;
+        this._updateAlwaysZoom();
+
+        global.screen.connect('monitors-changed', Lang.bind(this, this._updateAlwaysZoom));
+        global.screen.connect('notify::n-workspaces',
+                              Lang.bind(this, this._workspacesChanged));
+
+        Main.xdndHandler.connect('drag-begin', Lang.bind(this, function(){
+            this._alwaysZoomOut = true;
+        }));
+
+        Main.xdndHandler.connect('drag-end', Lang.bind(this, function(){
+            this._alwaysZoomOut = false;
+            this._updateAlwaysZoom();
+        }));
+
         this._switchWorkspaceNotifyId = 0;
 
         this._itemDragBeginId = 0;
+        this._itemDragCancelledId = 0;
         this._itemDragEndId = 0;
         this._windowDragBeginId = 0;
+        this._windowDragCancelledId = 0;
         this._windowDragEndId = 0;
     },
 
    show: function() {
+        this._zoomOut = this._alwaysZoomOut;
+        this._zoomFraction = this._alwaysZoomOut ? 1 : 0;
+        this._updateZoom();
+
         this._controls.show();
         this._thumbnailsBox.show();
 
         this._workspaces = [];
         for (let i = 0; i < global.screen.n_workspaces; i++) {
             let metaWorkspace = global.screen.get_workspace_by_index(i);
-            this._workspaces[i] = new Workspace.Workspace(metaWorkspace);
+            this._workspaces[i] = new Workspace.Workspace(metaWorkspace, this._monitorIndex);
         }
 
-        let rtl = (St.Widget.get_default_direction () == St.TextDirection.RTL);
-
-        let totalAllocation = this.actor.allocation;
-        let totalWidth = totalAllocation.x2 - totalAllocation.x1;
-        let totalHeight = totalAllocation.y2 - totalAllocation.y1;
-
-        let controlsVisible = this._controls.get_theme_node().get_length('visible-width');
-
-        totalWidth -= controlsVisible;
-
-        // Workspaces expect to have the same ratio as the screen, so take
-        // this into account when fitting the workspace into the available space
-        let width, height;
-        let totalRatio = totalWidth / totalHeight;
-        let wsRatio = global.screen_width / global.screen_height;
-        if (wsRatio > totalRatio) {
-            width = totalWidth;
-            height = Math.floor(totalWidth / wsRatio);
-        } else {
-            width = Math.floor(totalHeight * wsRatio);
-            height = totalHeight;
-        }
-
-        // Position workspaces in the available space
-        let [x, y] = this.actor.get_transformed_position();
-        x = Math.floor(x + Math.abs(totalWidth - width) / 2);
-        y = Math.floor(y + Math.abs(totalHeight - height) / 2);
-
-        if (rtl)
-            x += controlsVisible;
-
-        let newView = new WorkspacesView(width, height, x, y, this._workspaces);
-        this._updateZoomScale();
-
         if (this.workspacesView)
             this.workspacesView.destroy();
-        this.workspacesView = newView;
-
-        this._nWorkspacesNotifyId =
-            global.screen.connect('notify::n-workspaces',
-                                  Lang.bind(this, this._workspacesChanged));
+        this.workspacesView = new WorkspacesView(this._workspaces);
+        this._updateWorkspacesGeometry();
 
         this._restackedNotifyId =
             global.screen.connect('restacked',
@@ -711,30 +616,29 @@ WorkspacesDisplay.prototype = {
         if (this._itemDragBeginId == 0)
             this._itemDragBeginId = Main.overview.connect('item-drag-begin',
                                                           Lang.bind(this, this._dragBegin));
+        if (this._itemDragCancelledId == 0)
+            this._itemDragCancelledId = Main.overview.connect('item-drag-cancelled',
+                                                              Lang.bind(this, this._dragCancelled));
         if (this._itemDragEndId == 0)
             this._itemDragEndId = Main.overview.connect('item-drag-end',
                                                         Lang.bind(this, this._dragEnd));
         if (this._windowDragBeginId == 0)
             this._windowDragBeginId = Main.overview.connect('window-drag-begin',
                                                             Lang.bind(this, this._dragBegin));
+        if (this._windowDragCancelledId == 0)
+            this._windowDragCancelledId = Main.overview.connect('window-drag-cancelled',
+                                                            Lang.bind(this, this._dragCancelled));
         if (this._windowDragEndId == 0)
             this._windowDragEndId = Main.overview.connect('window-drag-end',
                                                           Lang.bind(this, this._dragEnd));
 
         this._onRestacked();
-        this._zoomOut = false;
-        this._zoomFraction = 0;
-        this._updateZoom();
     },
 
     hide: function() {
         this._controls.hide();
         this._thumbnailsBox.hide();
 
-        if (this._nWorkspacesNotifyId > 0) {
-            global.screen.disconnect(this._nWorkspacesNotifyId);
-            this._nWorkspacesNotifyId = 0;
-        }
         if (this._restackedNotifyId > 0){
             global.screen.disconnect(this._restackedNotifyId);
             this._restackedNotifyId = 0;
@@ -743,7 +647,11 @@ WorkspacesDisplay.prototype = {
             Main.overview.disconnect(this._itemDragBeginId);
             this._itemDragBeginId = 0;
         }
-        if (this._itemEndBeginId > 0) {
+        if (this._itemDragCancelledId > 0) {
+            Main.overview.disconnect(this._itemDragCancelledId);
+            this._itemDragCancelledId = 0;
+        }
+        if (this._itemDragEndId > 0) {
             Main.overview.disconnect(this._itemDragEndId);
             this._itemDragEndId = 0;
         }
@@ -751,6 +659,10 @@ WorkspacesDisplay.prototype = {
             Main.overview.disconnect(this._windowDragBeginId);
             this._windowDragBeginId = 0;
         }
+        if (this._windowDragCancelledId > 0) {
+            Main.overview.disconnect(this._windowDragCancelledId);
+            this._windowDragCancelledId = 0;
+        }
         if (this._windowDragEndId > 0) {
             Main.overview.disconnect(this._windowDragEndId);
             this._windowDragEndId = 0;
@@ -774,6 +686,28 @@ WorkspacesDisplay.prototype = {
         return this._zoomFraction;
     },
 
+    _updateAlwaysZoom: function()  {
+        // Always show the pager if workspaces are actually used,
+        // e.g. there are windows on more than one
+        this._alwaysZoomOut = global.screen.n_workspaces > 2;
+
+        if (this._alwaysZoomOut)
+            return;
+
+        let monitors = global.get_monitors();
+        let primary = global.get_primary_monitor();
+
+        /* Look for any monitor to the right of the primary, if there is
+         * one, we always keep zoom out, otherwise its hard to reach
+         * the thumbnail area without passing into the next monitor. */
+        for (let i = 0; i < monitors.length; i++) {
+            if (monitors[i].x >= primary.x + primary.width) {
+                this._alwaysZoomOut = true;
+                break;
+            }
+        }
+    },
+
     _getPreferredWidth: function (actor, forHeight, alloc) {
         // pass through the call in case the child needs it, but report 0x0
         this._controls.get_preferred_width(forHeight);
@@ -809,22 +743,48 @@ WorkspacesDisplay.prototype = {
         childBox.y2 = box.y2- box.y1;
         this._controls.allocate(childBox, flags);
 
-        this._updateZoomScale();
+        this._updateWorkspacesGeometry();
     },
 
-    _updateZoomScale: function() {
+    _updateWorkspacesGeometry: function() {
         if (!this.workspacesView)
             return;
 
-        let totalAllocation = this.actor.allocation;
-        let totalWidth = totalAllocation.x2 - totalAllocation.x1;
-        let totalHeight = totalAllocation.y2 - totalAllocation.y1;
+        let fullWidth = this.actor.allocation.x2 - this.actor.allocation.x1;
+        let fullHeight = this.actor.allocation.y2 - this.actor.allocation.y1;
 
-        let [controlsMin, controlsNatural] = this._controls.get_preferred_width(totalHeight);
+        let width = fullWidth;
+        let height = fullHeight;
+
+        let [controlsMin, controlsNatural] = this._controls.get_preferred_width(height);
         let controlsVisible = this._controls.get_theme_node().get_length('visible-width');
 
-        let zoomScale = (totalWidth - controlsNatural) / (totalWidth - controlsVisible);
-        this.workspacesView.setZoomScale(zoomScale);
+        let [x, y] = this.actor.get_transformed_position();
+
+        let rtl = (St.Widget.get_default_direction () == St.TextDirection.RTL);
+
+        let clipWidth = width - controlsVisible;
+        let clipHeight = (fullHeight / fullWidth) * clipWidth;
+        let clipX = rtl ? x + controlsVisible : x;
+        let clipY = y + (fullHeight - clipHeight) / 2;
+
+        this.workspacesView.setClipRect(clipX, clipY, clipWidth, clipHeight);
+
+        if (this._zoomOut) {
+            width -= controlsNatural;
+            if (rtl)
+                x += controlsNatural;
+        } else {
+            width -= controlsVisible;
+            if (rtl)
+                x += controlsVisible;
+        }
+
+        height = (fullHeight / fullWidth) * width;
+        let difference = fullHeight - height;
+        y += difference / 2;
+
+        this.workspacesView.setGeometry(x, y, width, height, difference);
     },
 
     _onRestacked: function() {
@@ -848,12 +808,18 @@ WorkspacesDisplay.prototype = {
         if (oldNumWorkspaces == newNumWorkspaces)
             return;
 
+        this._updateAlwaysZoom();
+        this._updateZoom();
+
+        if (this.workspacesView == null)
+            return;
+
         let lostWorkspaces = [];
         if (newNumWorkspaces > oldNumWorkspaces) {
             // Assume workspaces are only added at the end
             for (let w = oldNumWorkspaces; w < newNumWorkspaces; w++) {
                 let metaWorkspace = global.screen.get_workspace_by_index(w);
-                this._workspaces[w] = new Workspace.Workspace(metaWorkspace);
+                this._workspaces[w] = new Workspace.Workspace(metaWorkspace, this._monitorIndex);
             }
 
             this._thumbnailsBox.addThumbnails(oldNumWorkspaces, newNumWorkspaces - oldNumWorkspaces);
@@ -873,26 +839,26 @@ WorkspacesDisplay.prototype = {
             lostWorkspaces = this._workspaces.splice(removedIndex,
                                                      removedNum);
 
-            // Don't let the user try to select this workspace as it's
-            // making its exit.
-            for (let l = 0; l < lostWorkspaces.length; l++)
-                lostWorkspaces[l].setReactive(false);
+            for (let l = 0; l < lostWorkspaces.length; l++) {
+                lostWorkspaces[l].disconnectAll();
+                lostWorkspaces[l].destroy();
+            }
 
             this._thumbnailsBox.removeThumbmails(removedIndex, removedNum);
         }
 
         this.workspacesView.updateWorkspaces(oldNumWorkspaces,
-                                             newNumWorkspaces,
-                                             lostWorkspaces);
+                                             newNumWorkspaces);
     },
 
     _updateZoom : function() {
         if (Main.overview.animationInProgress)
             return;
 
-        let shouldZoom = this._controls.hover || this._inDrag;
+        let shouldZoom = this._alwaysZoomOut || this._controls.hover;
         if (shouldZoom != this._zoomOut) {
             this._zoomOut = shouldZoom;
+            this._updateWorkspacesGeometry();
 
             if (!this.workspacesView)
                 return;
@@ -902,10 +868,7 @@ WorkspacesDisplay.prototype = {
                                time: WORKSPACE_SWITCH_TIME,
                                transition: 'easeOutQuad' });
 
-            if (shouldZoom)
-                this.workspacesView.zoomOut();
-            else
-                this.workspacesView.zoomIn();
+            this.workspacesView.updateWindowPositions();
         }
     },
 
@@ -915,7 +878,23 @@ WorkspacesDisplay.prototype = {
 
     _dragBegin: function() {
         this._inDrag = true;
-        this._updateZoom();
+        this._cancelledDrag = false;
+        this._dragMonitor = {
+            dragMotion: Lang.bind(this, this._onDragMotion)
+        };
+        DND.addDragMonitor(this._dragMonitor);
+    },
+
+    _dragCancelled: function() {
+        this._cancelledDrag = true;
+        DND.removeDragMonitor(this._dragMonitor);
+    },
+
+    _onDragMotion: function(dragEvent) {
+        let controlsHovered = this._controls.contains(dragEvent.targetActor);
+        this._controls.set_hover(controlsHovered);
+
+        return DND.DragMotionResult.CONTINUE;
     },
 
     _dragEnd: function() {
