@@ -453,11 +453,13 @@ main (int argc, char **argv)
 {
   GOptionContext *ctx;
   GError *error = NULL;
+  int ecode;
 
   g_type_init ();
 
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
 
   ctx = meta_get_option_context ();
   g_option_context_add_main_entries (ctx, gnome_shell_options, GETTEXT_PACKAGE);
@@ -506,5 +508,13 @@ main (int argc, char **argv)
   /* Initialize the global object */
   shell_global_get ();
 
-  return meta_run ();
+  ecode = meta_run ();
+
+  if (g_getenv ("GNOME_SHELL_ENABLE_CLEANUP"))
+    {
+      g_printerr ("Doing final cleanup...\n");
+      g_object_unref (shell_global_get ());
+    }
+
+  return ecode;
 }
