@@ -2,8 +2,8 @@
 /* Generated data (by glib-mkenums) */
 
 #include "shell-enum-types.h"
-/* enumerations from "shell-app.h" */
-#include "shell-app.h"
+/* enumerations from "./shell-app.h" */
+#include "./shell-app.h"
 GType
 shell_app_state_get_type(void) {
   static GType enum_type_id = 0;
@@ -19,8 +19,24 @@ shell_app_state_get_type(void) {
     }
   return enum_type_id;
 }
-/* enumerations from "shell-global.h" */
-#include "shell-global.h"
+/* enumerations from "./shell-global.h" */
+#include "./shell-global.h"
+GType
+shell_stage_input_mode_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { SHELL_STAGE_INPUT_MODE_NONREACTIVE, "SHELL_STAGE_INPUT_MODE_NONREACTIVE", "nonreactive" },
+        { SHELL_STAGE_INPUT_MODE_NORMAL, "SHELL_STAGE_INPUT_MODE_NORMAL", "normal" },
+        { SHELL_STAGE_INPUT_MODE_FOCUSED, "SHELL_STAGE_INPUT_MODE_FOCUSED", "focused" },
+        { SHELL_STAGE_INPUT_MODE_FULLSCREEN, "SHELL_STAGE_INPUT_MODE_FULLSCREEN", "fullscreen" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static("ShellStageInputMode", values);
+    }
+  return enum_type_id;
+}
 GType
 shell_cursor_get_type(void) {
   static GType enum_type_id = 0;
@@ -38,24 +54,8 @@ shell_cursor_get_type(void) {
     }
   return enum_type_id;
 }
-GType
-shell_stage_input_mode_get_type(void) {
-  static GType enum_type_id = 0;
-  if (G_UNLIKELY (!enum_type_id))
-    {
-      static const GEnumValue values[] = {
-        { SHELL_STAGE_INPUT_MODE_NONREACTIVE, "SHELL_STAGE_INPUT_MODE_NONREACTIVE", "nonreactive" },
-        { SHELL_STAGE_INPUT_MODE_NORMAL, "SHELL_STAGE_INPUT_MODE_NORMAL", "normal" },
-        { SHELL_STAGE_INPUT_MODE_FOCUSED, "SHELL_STAGE_INPUT_MODE_FOCUSED", "focused" },
-        { SHELL_STAGE_INPUT_MODE_FULLSCREEN, "SHELL_STAGE_INPUT_MODE_FULLSCREEN", "fullscreen" },
-        { 0, NULL, NULL }
-      };
-      enum_type_id = g_enum_register_static("ShellStageInputMode", values);
-    }
-  return enum_type_id;
-}
-/* enumerations from "shell-mobile-providers.h" */
-#include "shell-mobile-providers.h"
+/* enumerations from "./shell-mobile-providers.h" */
+#include "./shell-mobile-providers.h"
 GType
 shell_mobile_access_method_type_get_type(void) {
   static GType enum_type_id = 0;
