@@ -8,16 +8,20 @@
 
 G_BEGIN_DECLS
 
-/* enumerations from "shell-app.h" */
+/* enumerations from "./shell-app.h" */
 GType shell_app_state_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_APP_STATE (shell_app_state_get_type())
 
-/* enumerations from "shell-global.h" */
+/* enumerations from "./shell-global.h" */
+GType shell_stage_input_mode_get_type (void) G_GNUC_CONST;
+#define SHELL_TYPE_STAGE_INPUT_MODE (shell_stage_input_mode_get_type())
+
 GType shell_cursor_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_CURSOR (shell_cursor_get_type())
 
-GType shell_stage_input_mode_get_type (void) G_GNUC_CONST;
-#define SHELL_TYPE_STAGE_INPUT_MODE (shell_stage_input_mode_get_type())
+/* enumerations from "./shell-mobile-providers.h" */
+GType shell_mobile_access_method_type_get_type (void) G_GNUC_CONST;
+#define SHELL_TYPE_MOBILE_ACCESS_METHOD_TYPE (shell_mobile_access_method_type_get_type())
 
 G_END_DECLS
 
