@@ -22,12 +22,14 @@
 #include <gio/gio.h>
 #include <gjs/gjs-module.h>
 #include <girepository.h>
-#include <libxml/parser.h>
-#include <libxml/tree.h>
-#include <libxml/xmlmemory.h>
 #include <meta/display.h>
 #include <meta/util.h>
 
+/* Memory report bits */
+#ifdef HAVE_MALLINFO
+#include <malloc.h>
+#endif
+
 #include "shell-enum-types.h"
 #include "shell-global-private.h"
 #include "shell-jsapi-compat-private.h"
@@ -37,6 +39,8 @@
 #include "shell-wm.h"
 #include "st.h"
 
+static ShellGlobal *the_object = NULL;
+
 static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data);
 
 struct _ShellGlobal {
@@ -65,8 +69,6 @@ struct _ShellGlobal {
 
   GdkWindow    *stage_window;
 
-  gint last_change_screen_width, last_change_screen_height;
-
   guint work_count;
   GSList *leisure_closures;
   guint leisure_function_id;
@@ -240,9 +242,6 @@ shell_global_init (ShellGlobal *global)
 
   global->input_mode = SHELL_STAGE_INPUT_MODE_NORMAL;
 
-  global->last_change_screen_width = 0;
-  global->last_change_screen_height = 0;
-
   ca_context_create (&global->sound_context);
   ca_context_change_props (global->sound_context, CA_PROP_APPLICATION_NAME, PACKAGE_NAME, CA_PROP_APPLICATION_ID, "org.gnome.Shell", NULL);
   ca_context_open (global->sound_context);
@@ -257,6 +256,20 @@ shell_global_init (ShellGlobal *global)
   g_strfreev (search_path);
 }
 
+static void
+shell_global_finalize (GObject *object)
+{
+  ShellGlobal *global = SHELL_GLOBAL (object);
+
+  g_object_unref (global->js_context);
+  gtk_widget_destroy (GTK_WIDGET (global->grab_notifier));
+  g_object_unref (global->settings);
+
+  the_object = NULL;
+
+  G_OBJECT_CLASS(shell_global_parent_class)->finalize (object);
+}
+
 static void
 shell_global_class_init (ShellGlobalClass *klass)
 {
@@ -264,6 +277,7 @@ shell_global_class_init (ShellGlobalClass *klass)
 
   gobject_class->get_property = shell_global_get_property;
   gobject_class->set_property = shell_global_set_property;
+  gobject_class->finalize = shell_global_finalize;
 
   /* Emitted from gnome-shell-plugin.c during event handling */
   shell_global_signals[XDND_POSITION_CHANGED] =
@@ -417,31 +431,6 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         G_PARAM_READABLE));
 }
 
-/**
- * shell_clutter_texture_set_from_pixbuf: 
- * texture: #ClutterTexture to be modified
- * pixbuf: #GdkPixbuf to set as an image for #ClutterTexture
- *
- * Convenience function for setting an image for #ClutterTexture based on #GdkPixbuf.
- * Copied from an example posted by hp in this thread http://mail.gnome.org/archives/gtk-devel-list/2008-September/msg00218.html
- *
- * Return value: %TRUE on success, %FALSE on failure
- */
-gboolean
-shell_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
-                                       GdkPixbuf      *pixbuf)
-{
-    return clutter_texture_set_from_rgb_data (texture,
-                                              gdk_pixbuf_get_pixels (pixbuf),
-                                              gdk_pixbuf_get_has_alpha (pixbuf),
-                                              gdk_pixbuf_get_width (pixbuf),
-                                              gdk_pixbuf_get_height (pixbuf),
-                                              gdk_pixbuf_get_rowstride (pixbuf),
-                                              gdk_pixbuf_get_has_alpha (pixbuf)
-                                              ? 4 : 3,
-                                              0, NULL);
-}
-
 /**
  * shell_global_get:
  *
@@ -452,8 +441,6 @@ shell_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
 ShellGlobal *
 shell_global_get (void)
 {
-  static ShellGlobal *the_object = NULL;
-
   if (!the_object)
     the_object = g_object_new (SHELL_TYPE_GLOBAL, 0);
 
@@ -708,23 +695,6 @@ shell_global_get_window_actors (ShellGlobal *global)
   return meta_plugin_get_window_actors (global->plugin);
 }
 
-static gboolean
-update_screen_size (gpointer data)
-{
-  int width, height;
-  ShellGlobal *global = SHELL_GLOBAL (data);
-
-  meta_plugin_query_screen_size (global->plugin, &width, &height);
-
-  if (global->last_change_screen_width == width && global->last_change_screen_height == height)
-    return FALSE;
-
-  global->last_change_screen_width = width;
-  global->last_change_screen_height = height;
-
-  return FALSE;
-}
-
 static void
 global_stage_notify_width (GObject    *gobject,
                            GParamSpec *pspec,
@@ -733,11 +703,6 @@ global_stage_notify_width (GObject    *gobject,
   ShellGlobal *global = SHELL_GLOBAL (data);
 
   g_object_notify (G_OBJECT (global), "screen-width");
-
-  meta_later_add (META_LATER_BEFORE_REDRAW,
-                  update_screen_size,
-                  global,
-                  NULL);
 }
 
 static void
@@ -748,11 +713,6 @@ global_stage_notify_height (GObject    *gobject,
   ShellGlobal *global = SHELL_GLOBAL (data);
 
   g_object_notify (G_OBJECT (global), "screen-height");
-
-  meta_later_add (META_LATER_BEFORE_REDRAW,
-                  update_screen_size,
-                  global,
-                  NULL);
 }
 
 static void
@@ -791,7 +751,6 @@ _shell_global_set_plugin (ShellGlobal *global,
                     G_CALLBACK (global_stage_notify_width), global);
   g_signal_connect (stage, "notify::height",
                     G_CALLBACK (global_stage_notify_height), global);
-  update_screen_size (global);
 
   g_signal_connect (stage, "paint",
                     G_CALLBACK (global_stage_before_paint), global);
@@ -860,30 +819,6 @@ shell_global_end_modal (ShellGlobal *global,
   meta_plugin_end_modal (global->plugin, timestamp);
 }
 
-/**
- * shell_global_display_is_grabbed
- * @global: a #ShellGlobal
- *
- * Determines whether Mutter currently has a grab (keyboard or mouse or
- * both) on the display. This could be the result of a current window
- * management operation like a window move, or could be from
- * shell_global_begin_modal().
- *
- * This function is useful to for ad-hoc checks to avoid over-grabbing
- * the Mutter grab a grab from GTK+. Longer-term we might instead want a
- * mechanism to make Mutter use GDK grabs instead of raw XGrabPointer().
- *
- * Return value: %TRUE if Mutter has a grab on the display
- */
-gboolean
-shell_global_display_is_grabbed (ShellGlobal *global)
-{
-  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
-  MetaDisplay *display = meta_screen_get_display (screen);
-
-  return meta_display_get_grab_op (display) != META_GRAB_OP_NONE;
-}
-
 /**
  * shell_global_create_pointer_barrier
  * @global: a #ShellGlobal
@@ -1134,149 +1069,6 @@ shell_global_reexec_self (ShellGlobal *global)
   g_ptr_array_free (arr, TRUE);
 }
 
-/**
- * shell_global_breakpoint:
- * @global: A #ShellGlobal
- *
- * Using G_BREAKPOINT(), interrupt the current process.  This is useful
- * in conjunction with a debugger such as gdb.
- */
-void
-shell_global_breakpoint (ShellGlobal *global)
-{
-  G_BREAKPOINT ();
-}
-
-/**
- * shell_global_parse_search_provider:
- * @global: A #ShellGlobal
- * @data: description of provider
- * @name: (out): location to store a display name
- * @url: (out): location to store template of url
- * @langs: (out) (transfer full) (element-type utf8): list of supported languages
- * @icon_data_uri: (out): location to store uri
- * @error: location to store GError
- *
- * Returns: %TRUE on success
- */
-gboolean
-shell_global_parse_search_provider (ShellGlobal   *global,
-                                    const char    *data,
-                                    char         **name,
-                                    char         **url,
-                                    GList        **langs,
-                                    char         **icon_data_uri,
-                                    GError       **error)
-{
-  xmlDocPtr doc = xmlParseMemory (data, strlen(data));
-  xmlNode *root;
-
-  *name = NULL;
-  *url = NULL;
-  *icon_data_uri = NULL;
-  *langs = NULL;
-
-  if (!doc)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Malformed xml");
-      return FALSE;
-    }
-
-  root = xmlDocGetRootElement (doc);
-  if (root && root->name && xmlStrcmp (root->name, (const xmlChar *)"OpenSearchDescription") == 0)
-    {
-      xmlNode *child;
-      for (child = root->children; child; child = child->next)
-        {
-            if (!child->name)
-              continue;
-            if (xmlStrcmp (child->name, (const xmlChar *)"Language") == 0)
-              {
-                xmlChar *val = xmlNodeListGetString(doc, child->xmlChildrenNode, 1);
-                if (!val)
-                  continue;
-                *langs = g_list_append (*langs, g_strdup ((char *)val));
-                xmlFree (val);
-              }
-            if (!*name && xmlStrcmp (child->name, (const xmlChar *)"ShortName") == 0)
-              {
-                xmlChar *val = xmlNodeListGetString(doc, child->xmlChildrenNode, 1);
-                *name = g_strdup ((char *)val);
-                xmlFree (val);
-              }
-            if (!*icon_data_uri && xmlStrcmp (child->name, (const xmlChar *)"Image") == 0)
-              {
-                xmlChar *val = xmlNodeListGetString(doc, child->xmlChildrenNode, 1);
-                if (val)
-                  *icon_data_uri = g_strdup ((char *)val);
-                xmlFree (val);
-              }
-            if (!*url && xmlStrcmp (child->name, (const xmlChar *)"Url") == 0)
-              {
-                xmlChar *template;
-                xmlChar *type;
-
-                type = xmlGetProp(child, (const xmlChar *)"type");
-                if (!type)
-                  continue;
-
-                if (xmlStrcmp (type, (const xmlChar *)"text/html") != 0)
-                  {
-                    xmlFree (type);
-                    continue;
-                  }
-                xmlFree (type);
-
-                template = xmlGetProp(child, (const xmlChar *)"template");
-                if (!template)
-                  continue;
-                *url = g_strdup ((char *)template);
-                xmlFree (template);
-              }
-        }
-    }
-  else
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Invalid OpenSearch document");
-      xmlFreeDoc (doc);
-      return FALSE;
-    }
-  xmlFreeDoc (doc);
-  if (*icon_data_uri && *name && *url)
-    return TRUE;
-
-  if (*icon_data_uri)
-    g_free (*icon_data_uri);
-  else
-    g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 "search provider doesn't have icon");
-
-  if (*name)
-    g_free (*name);
-  else if (error && !*error)
-    g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 "search provider doesn't have ShortName");
-
-  if (*url)
-    g_free (*url);
-  else if (error && !*error)
-    g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                 "search provider doesn't have template for url");
-
-  if (*langs)
-    {
-      g_list_foreach (*langs, (GFunc)g_free, NULL);
-      g_list_free (*langs);
-    }
-
-  *url = NULL;
-  *name = NULL;
-  *icon_data_uri = NULL;
-  *langs = NULL;
-
-  return FALSE;
-}
-
 /**
  * shell_global_gc:
  * @global: A #ShellGlobal
@@ -1306,6 +1098,38 @@ shell_global_maybe_gc (ShellGlobal *global)
   gjs_context_maybe_gc (global->js_context);
 }
 
+/**
+ * shell_global_get_memory_info:
+ * @global:
+ * @meminfo: (out caller-allocates): Output location for memory information
+ *
+ * Load process-global data about memory usage.
+ */
+void
+shell_global_get_memory_info (ShellGlobal        *global,
+                              ShellMemoryInfo    *meminfo)
+{
+  JSContext *context;
+
+  memset (meminfo, 0, sizeof (meminfo));
+#ifdef HAVE_MALLINFO
+  {
+    struct mallinfo info = mallinfo ();
+    meminfo->glibc_uordblks = info.uordblks;
+  }
+#endif
+
+  context = gjs_context_get_native_context (global->js_context);
+
+  meminfo->js_bytes = JS_GetGCParameter (JS_GetRuntime (context), JSGC_BYTES);
+
+  meminfo->gjs_boxed = (unsigned int) gjs_counter_boxed.value;
+  meminfo->gjs_gobject = (unsigned int) gjs_counter_object.value;
+  meminfo->gjs_function = (unsigned int) gjs_counter_function.value;
+  meminfo->gjs_closure = (unsigned int) gjs_counter_closure.value;
+}
+
+
 /**
  * shell_global_notify_error:
  * @global: a #ShellGlobal
@@ -1369,54 +1193,6 @@ void shell_global_init_xdnd (ShellGlobal *global)
                   32, PropModeReplace, (const unsigned char *)&stage_win, 1);
 }
 
-/**
- * shell_global_format_time_relative_pretty:
- * @global:
- * @delta: Time in seconds since the current time
- * @text: (out): Relative human-consumption-only time string
- * @next_update: (out): Time in seconds until we should redisplay the time
- *
- * Format a time value for human consumption only.  The passed time
- * value is a delta in terms of seconds from the current time.
- * This function needs to be in C because of its use of ngettext() which
- * is not accessible from JavaScript.
- */
-void
-shell_global_format_time_relative_pretty (ShellGlobal *global,
-                                          guint        delta,
-                                          char       **text,
-                                          guint       *next_update)
-{
-#define MINUTE (60)
-#define HOUR (MINUTE*60)
-#define DAY (HOUR*24)
-#define WEEK (DAY*7)
-  if (delta < MINUTE) {
-    *text = g_strdup (_("Less than a minute ago"));
-    *next_update = MINUTE - delta;
-   } else if (delta < HOUR) {
-     *text = g_strdup_printf (dngettext (GETTEXT_PACKAGE,
-                                         "%d minute ago", "%d minutes ago",
-                                         delta / MINUTE), delta / MINUTE);
-     *next_update = MINUTE - (delta % MINUTE);
-   } else if (delta < DAY) {
-     *text = g_strdup_printf (dngettext (GETTEXT_PACKAGE,
-                                         "%d hour ago", "%d hours ago",
-                                         delta / HOUR), delta / HOUR);
-     *next_update = HOUR - (delta % HOUR);
-   } else if (delta < WEEK) {
-     *text = g_strdup_printf (dngettext (GETTEXT_PACKAGE,
-                                         "%d day ago", "%d days ago",
-                                         delta / DAY), delta / DAY);
-     *next_update = DAY - (delta % DAY);
-   } else {
-     *text = g_strdup_printf (dngettext (GETTEXT_PACKAGE,
-                                         "%d week ago", "%d weeks ago",
-                                         delta / WEEK), delta / WEEK);
-     *next_update = WEEK - (delta % WEEK);
-   }
-}
-
 /**
  * shell_global_get_monitors:
  * @global: the #ShellGlobal
@@ -1517,8 +1293,7 @@ shell_global_get_focus_monitor (ShellGlobal  *global)
         }
     }
 
-  meta_screen_get_monitor_geometry (screen, 0, &rect);
-  return meta_rectangle_copy (&rect);
+  return shell_global_get_primary_monitor (global);
 }
 
 /**
@@ -1587,25 +1362,6 @@ shell_global_sync_pointer (ShellGlobal *global)
   clutter_event_put ((ClutterEvent *)&event);
 }
 
-/**
- * shell_get_event_state:
- * @event: a #ClutterEvent
- *
- * Gets the current state of the event (the set of modifier keys that
- * are pressed down). Thhis is a wrapper around
- * clutter_event_get_state() that strips out any un-declared modifier
- * flags, to make gjs happy; see
- * https://bugzilla.gnome.org/show_bug.cgi?id=597292.
- *
- * Return value: the state from the event
- */
-ClutterModifierType
-shell_get_event_state (ClutterEvent *event)
-{
-  ClutterModifierType state = clutter_event_get_state (event);
-  return state & CLUTTER_MODIFIER_MASK;
-}
-
 /**
  * shell_global_get_settings:
  * @global: A #ShellGlobal
@@ -1620,59 +1376,6 @@ shell_global_get_settings (ShellGlobal *global)
   return global->settings;
 }
 
-static void
-shell_popup_menu_position_func (GtkMenu   *menu,
-                                int       *x,
-                                int       *y,
-                                gboolean  *push_in,
-                                gpointer   user_data)
-{
-  *x = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu), "shell-menu-x"));
-  *y = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu), "shell-menu-y"));
-}
-
-/**
- * shell_popup_menu:
- * @menu: a #GtkMenu
- * @button: mouse button that triggered the menu
- * @time: timestamp of event that triggered the menu
- * @menu_x: x coordinate to display the menu at
- * @menu_y: y coordinate to display the menu at
- *
- * Wraps gtk_menu_popup(), but using @menu_x, @menu_y for the location
- * rather than needing a callback.
- **/
-void
-shell_popup_menu (GtkMenu *menu, int button, guint32 time,
-                  int menu_x, int menu_y)
-{
-  g_object_set_data (G_OBJECT (menu), "shell-menu-x", GINT_TO_POINTER (menu_x));
-  g_object_set_data (G_OBJECT (menu), "shell-menu-y", GINT_TO_POINTER (menu_y));
-
-  gtk_menu_popup (menu, NULL, NULL, shell_popup_menu_position_func, NULL,
-                  button, time);
-}
-
-/**
- * shell_write_string_to_stream:
- * @stream: a #GOutputStream
- * @str: a UTF-8 string to write to @stream
- * @error: location to store GError
- *
- * Write a string to a GOutputStream as UTF-8. This is a workaround
- * for not having binary buffers in GJS.
- *
- * Return value: %TRUE if write succeeded
- */
-gboolean
-shell_write_string_to_stream (GOutputStream *stream,
-                              const char    *str,
-                              GError       **error)
-{
-  return g_output_stream_write_all (stream, str, strlen (str),
-                                    NULL, NULL, error);
-}
-
 /**
  * shell_global_get_current_time:
  * @global: A #ShellGlobal
@@ -1746,73 +1449,6 @@ shell_global_create_app_launch_context (ShellGlobal *global)
   return (GAppLaunchContext *)context;
 }
 
-/**
- * shell_global_set_property_mutable:
- * @global: the #ShellGlobal
- * @object: the "path" to a JS object, starting from the root object.
- *  (Eg, "global.stage" or "imports.gi.Gtk.Window.prototype")
- * @property: a property on @object
- * @mutable: %TRUE or %FALSE
- *
- * If @mutable is %TRUE, this clears the "permanent" and "readonly" flags
- * on @property of @object. If @mutable is %FALSE, it sets them.
- *
- * You can use this to make it possible to modify properties that
- * would otherwise be read-only from JavaScript.
- *
- * Return value: success or failure.
- */
-gboolean
-shell_global_set_property_mutable (ShellGlobal *global,
-                                   const char  *object,
-                                   const char  *property,
-                                   gboolean     mutable)
-{
-  JSContext *context = gjs_context_get_native_context (global->js_context);
-  char **parts;
-  JSObject *obj;
-  jsval val = JSVAL_VOID;
-  int i;
-  jsuint attrs;
-  JSBool found;
-
-  JS_BeginRequest (context);
-  JS_AddValueRoot (context, &val);
-
-  parts = g_strsplit (object, ".", -1);
-  obj = JS_GetGlobalObject (context);
-  for (i = 0; parts[i]; i++)
-    {
-      if (!JS_GetProperty (context, obj, parts[i], &val))
-        {
-          g_strfreev (parts);
-          goto out_error;
-        }
-      obj = JSVAL_TO_OBJECT (val);
-    }
-  g_strfreev (parts);
-
-  if (!JS_GetPropertyAttributes (context, obj, property, &attrs, &found) || !found)
-    goto out_error;
-
-  if (mutable)
-    attrs &= ~(JSPROP_PERMANENT | JSPROP_READONLY);
-  else
-    attrs |= (JSPROP_PERMANENT | JSPROP_READONLY);
-
-  if (!JS_SetPropertyAttributes (context, obj, property, attrs, &found))
-    goto out_error;
-
-  JS_RemoveValueRoot (context, &val);
-  JS_EndRequest (context);
-  return TRUE;
- out_error:
-  gjs_log_exception (context, NULL);
-  JS_RemoveValueRoot (context, &val);
-  JS_EndRequest (context);
-  return FALSE;
-}
-
 typedef struct
 {
   ShellLeisureFunction func;
@@ -2049,66 +1685,6 @@ gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
     return FALSE;
 }
 
-/**
- * ShellGetTpContactCb:
- * @connection: The connection
- * @contacts: (element-type TelepathyGLib.Contact): List of contacts
- * @failed: Array of failed contacts
- */
-
-static void
-shell_global_get_tp_contacts_cb (TpConnection *self,
-                                 guint n_contacts,
-                                 TpContact * const *contacts,
-                                 guint n_failed,
-                                 const TpHandle *failed,
-                                 const GError *error,
-                                 gpointer user_data,
-                                 GObject *weak_object)
-{
-  int i;
-  GList *contact_list = NULL;
-  for (i = 0; i < n_contacts; i++) {
-      contact_list = g_list_append(contact_list, contacts[i]);
-  }
-
-  TpHandle *failed_list = g_new0 (TpHandle, n_failed + 1);
-  memcpy(failed_list, failed, n_failed);
-
-  ((ShellGetTpContactCb)user_data)(self, contact_list, failed_list);
-}
-
-
-/**
- * shell_get_tp_contacts:
- * @self: A connection, which must be ready
- * @n_handles: Number of handles in handles
- * @handles: (array length=n_handles) (element-type uint): Array of handles
- * @n_features: Number of features in features
- * @features: (array length=n_features) (allow-none) (element-type uint):
- *  Array of features
- * @callback: (scope async): User callback to run when the contacts are ready
- *
- * Wrap tp_connection_get_contacts_by_handle so we can transform the array
- * into a null-terminated one, which gjs can handle.
- * We send the original callback to tp_connection_get_contacts_by_handle as
- * user_data, and we have our own function as callback, which does the
- * transforming.
- */
-void
-shell_get_tp_contacts (TpConnection *self,
-                       guint n_handles,
-                       const TpHandle *handles,
-                       guint n_features,
-                       const TpContactFeature *features,
-                       ShellGetTpContactCb callback)
-{
-  tp_connection_get_contacts_by_handle(self, n_handles, handles,
-                                       n_features, features,
-                                       shell_global_get_tp_contacts_cb,
-                                       callback, NULL, NULL);
-}
-
 /**
  * shell_global_launch_calendar_server:
  * @global: The #ShellGlobal.
@@ -2162,103 +1738,3 @@ shell_global_launch_calendar_server (ShellGlobal *global)
 
   g_free (calendar_server_exe);
 }
-
-static void
-shell_global_get_self_contact_features_cb (TpConnection *connection,
-                                           guint n_contacts,
-                                           TpContact * const *contacts,
-                                           const GError *error,
-                                           gpointer user_data,
-                                           GObject *weak_object)
-{
-  if (error != NULL) {
-    g_print ("Failed to upgrade self contact: %s", error->message);
-    return;
-  }
-  ((ShellGetSelfContactFeaturesCb)user_data)(connection, *contacts);
-}
-
-/**
- * shell_get_self_contact_features:
- * @self: A connection, which must be ready
- * @n_features: Number of features in features
- * @features: (array length=n_features) (allow-none) (element-type uint):
- *  Array of features
- * @callback: (scope async): User callback to run when the contact is ready
- * 
- * Wrap tp_connection_upgrade_contacts due to the lack of support for
- * proper arrays arguments in GJS.
- */
-void
-shell_get_self_contact_features (TpConnection *self,
-                                 guint n_features,
-                                 const TpContactFeature *features,
-                                 ShellGetSelfContactFeaturesCb callback)
-{
-  TpContact *self_contact = tp_connection_get_self_contact (self);
-
-  tp_connection_upgrade_contacts (self, 1, &self_contact,
-                                  n_features, features,
-                                  shell_global_get_self_contact_features_cb,
-                                  callback, NULL, NULL);
-}
-
-/**
- * shell_get_contact_events:
- * @log_manager: A #TplLogManager
- * @account: A #TpAccount
- * @entity: A #TplEntity
- * @num_events: The number of events to retrieve
- * @callback: (scope async): User callback to run when the contact is ready
- * 
- * Wrap tpl_log_manager_get_filtered_events_async because gjs cannot support
- * multiple callbacks in the same function call.
- */
-void
-shell_get_contact_events (TplLogManager *log_manager,
-                          TpAccount *account,
-                          TplEntity *entity,
-                          guint num_events,
-                          GAsyncReadyCallback callback)
-{
-  tpl_log_manager_get_filtered_events_async (log_manager,
-                                             account,
-                                             entity,
-                                             TPL_EVENT_MASK_TEXT,
-                                             num_events,
-                                             NULL, NULL,
-                                             callback, NULL);
-}
-
-
-/**
- * shell_get_file_contents_utf8_sync:
- * @path: UTF-8 encoded filename path
- * @error: a #GError
- *
- * Synchronously load the contents of a file as a NUL terminated
- * string, validating it as UTF-8.  Embedded NUL characters count as
- * invalid content.
- *
- * Returns: (transfer full): File contents
- */
-char *
-shell_get_file_contents_utf8_sync (const char *path,
-                                   GError    **error)
-{
-  char *contents;
-  gsize len;
-  if (!g_file_get_contents (path, &contents, &len, error))
-    return NULL;
-  if (!g_utf8_validate (contents, len, NULL))
-    {
-      g_free (contents);
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_FAILED,
-                   "File %s contains invalid UTF-8",
-                   path);
-      return NULL;
-    }
-  return contents;
-}
